/*
	Title:	Converter tool for creating graph colouring problems from round robin sports tournaments
	file:	rrob_to_gcol.cpp
	Does:	takes as progam arguments an (even) integer specifying the number of teams, an output file and a 0 or 1 
			asking if we'd like screen output or not
	Source: Rhydian Lewis's Homepage at Prifysgol Caerdydd/Cardiff University
	Author:	Rhyd Lewis (November 2007) 
	email:	LewisR9@cf.ac.uk

	Compilation: Ansi C++ Code

	Inputs: an even integer
			output file name
			a seed

	Outputs: the generated graph in DIMACS standard format

	Copyright: Rhydian Lewis. Permission is granted to
		use and distribute freely for non-commercial purposes
		with appropriate attribution.
	
	Warranty: There is no warranty. Feel free to update this code yourself...
	
	Special Instructions: If any updates are made that improve this code then please let me know

	Cheers and Iachi Da!!!
*/


#include "rrob_to_gcol.h"
#include "converterfunctions.h"
#include <string.h>

int main (int argc, char *argv[])
{
	//PROGRAM VARIABLES
	//n refers to the number of teams
	int i, n, choice1, choice2;
	bool doubleRR = false, isMirrored = false;
	bool doScreenOutput = false;
	bool doingOtherConstraints = false, perfectSolution = false, writeComments = false;
	double unavailProb, preAssignProb;
	char solFileName[50];

	if(argc != 4){
		cout<<"USAGE numTeams(n) outputfile r_seed\n";
		cout<<"Note: there is very little input error checking in this program so be careful what you type...\n";
		exit(1);
	}

	//READ IN THE PROGRAM ARGUMENTS
	n = atoi(argv[1]);
	if(n%2 !=0){ cout << "Error: Number of teams should be even\n";	exit(1);}

	cout<<"Type of Round Robin: (single/double (0/1)) : ";
	cin>>choice1;
	
	if(choice1 == 1) {
		doubleRR = true;
		//cout<<"Making double round robin. Should this be mirrored? (0/1) :";
		//cin>>choice2;
		//if(choice2==1)
		//	isMirrored = true;
	}

	int rSeed = atoi(argv[3]); 
	srand(rSeed);

	if(!doubleRR && isMirrored){
		cout<<"Cannot have a mirrored single RR. Exiting...\n";
		exit(1);
	}

	//Sort out the output file extension etc.
	ofstream outStream;
	char outputname[20];
	strcpy(outputname,argv[2]);
	if(doubleRR){
		if(isMirrored)
			strcat(outputname, ".mir");
		else
			strcat(outputname, ".dob");
	}
	else
		strcat(outputname, ".sin");
	outStream.open(outputname);
	if(outStream.fail()) {
		cout << "Error opening output file\n"; exit(1);}

	cout<<"Output to screen? (0/1) : ";
	cin>>choice1;
	if(choice1 == 1) 
		doScreenOutput = true;
	else doScreenOutput = false;

	cout<<"Add further constraints? (0/1) : ";
	cin>>choice1;
	if(choice1 == 1) {
		doingOtherConstraints = true;
		cout<<"Do you want a guaranteed perfect solution (0/1) : ";
		cin>>choice2;
		if(choice2 == 1){
			perfectSolution = true;
			cout<<"Input the solution filename (with extension) : ";
			cin >> solFileName;
		}
		cout<<"Enter proportion of matches that should be preassigned [0,...,1] : ";
		cin >> preAssignProb;
		if( preAssignProb < 0.0 ||  preAssignProb > 1.0){
			cout<<"Error in range. Exiting... ";
			exit(1);
		}
		cout<<"Enter proportion of match/round combinations that should be disallowed [0,...,1] : ";
		cin >> unavailProb;
		if( unavailProb < 0.0 ||  unavailProb > 1.0){
			cout<<"Error in range. Exiting... ";
			exit(1);
		}
	}
	
	cout<<"Output comments in output file? (O/1) >> : ";
        cin>>choice2;
	if(choice2 == 1){
		writeComments = true;
	}
	else writeComments = false;


	//**********************************************************************
	//MAKE WELCOME MESSAGE IF APPROPRIATE
	if(doScreenOutput){
		cout<<"\nRHYD LEWIS'S ROUND ROBIN TO GRAPH COLOURING CONVERTER. WELCOME!\n";
		cout<<"-----------------------------------------------------------------\n\n";
		if(doubleRR){
			if(isMirrored)
				cout<<"Making a graph corresponding to a DOUBLE, SYMMETRICAL round-robin tournament\n\n";
			else
				cout<<"Making a graph corresponding to a DOUBLE, ASSYMETRICAL round-robin tournament\n\n";
		}
		else cout<<"Making a graph corresponding to a SINGLE round-robin tournament\n\n";
	}
		
	int numMatchNodes;
	if(doubleRR)numMatchNodes = n*(n-1);
	else numMatchNodes = (n*(n-1))/2;
	
	bool **adjacent = new bool* [numMatchNodes];
	for(i=0; i<numMatchNodes; i++) adjacent[i] = new bool [numMatchNodes];
	int *hometeam = new int [numMatchNodes];
	int *awayteam = new int [numMatchNodes];

	//Now convert by filling in the adjacency matrix (this matrix corresponds to the constraints imposed in
	//a normal rr. (i.e. extra constraints are not considered here. Note that a perfect solution will always
	//exist for such a problem)
	convertToGCol(n,adjacent,numMatchNodes,doubleRR,doScreenOutput,hometeam,awayteam);
	int numMatchEdges = calcNumEdges(adjacent, numMatchNodes);

	//Now make the output file. Note that extra constraints are added at this point
	makeOutput(outputname, outStream, numMatchNodes, adjacent, numMatchEdges, n, doubleRR, hometeam, awayteam, doingOtherConstraints, isMirrored, perfectSolution, preAssignProb, unavailProb, solFileName, rSeed, writeComments, doScreenOutput);
	
	//----------------------------------------------------------------------
	return 0;
}


